/*****
*
*   Contour.js
*
*   written by Kevin Lindsey
*   copyright 2001-2002, Kevin Lindsey
*
*   Based on Sven Moen's article "Drawing Dynamic Trees"
*
*****/

/*****
*
*   main constructor
*
*****/
function Contour() {
    this.lower_head = null;
    this.lower_tail = null;
    this.upper_head = null;
    this.upper_tail = null;
}


/*****
*
*   clone
*
*****/
Contour.prototype.clone = function() {
    var clone = new Contour();

    clone.lower_head = this.lower_head;
    clone.lower_tail = this.lower_tail;
    clone.upper_head = this.upper_head;
    clone.upper_tail = this.upper_tail;

    return clone;
};


/*****
*
*   merge
*
*****/
Contour.prototype.merge = function(that) {
    var point  = new Point2D(0,0);
    var height = 0;
    var upper  = this.lower_head;
    var lower  = that.upper_head;

    while ( upper != null && lower != null ) {
        var d = point.offset(lower.offset, upper.offset);

        point.rmoveto(0, d);
        height += d;

        if ( point.x + lower.offset.x <= upper.offset.x ) {
            point.rmoveto(lower.offset.x, lower.offset.y);
            lower = lower.next;
        } else {
            point.rmoveto(-upper.offset.x, -upper.offset.y);
            upper = upper.next;
        }
    }

    if ( lower != null ) {
        var b = this.upper_tail.bridge(new Point2D(0,0), lower, point);

        this.upper_tail = ( b.next != null ) ? that.upper_tail : b;
        this.lower_tail = that.lower_tail;
    } else {
        var b = that.lower_tail.bridge(point, upper, new Point2D(0,0));

        if ( b.next == null ) {
            this.lower_tail = b;
        }
    }

    this.lower_head = that.lower_head;

    return height;
};

