/*****
*
*	Slider.js
*	Copyright 2000, Kevin Lindsey
*
*	Based on code by Dr. Stefan Goessner
*
*	Uses Node_Builder
*
*****/

Slider.VERSION = 2.0;
Slider.sliders = new Array();

/*****
*
*	Slider Constructor
*
*****/
function Slider(x, y, size, direction, callback, parent, body_id, thumb_id) {
	this.x         = x;
	this.y         = y;
	this.size      = size;
	this.direction = direction;
	this.min       = 0;
	this.max       = size;
	this.value     = 0;
	this.active    = false;

	this.parent    = parent;
	this.body      = null;
	this.thumb     = null;

	this.callback  = callback;

	this.make_controls(body_id, thumb_id);
	Slider.sliders[Slider.sliders.length] = this;
};


/*****
*
*	make_controls
*
*****/
Slider.prototype.make_controls = function(body_id, thumb_id) {
    var SVGDoc   = this.parent.ownerDocument;
	var serial   = "_slider_" + Slider.sliders.length;
	var trans    = "translate(" + this.x + "," + this.y + ")";
	var rotate   = "rotate(" + this.direction + ")";
    var body, thumb;

    body = SVGDoc.getElementById(body_id).cloneNode(true);
    body.setAttributeNS(null, "id", serial);
    body.setAttributeNS(null, "transform", trans + " " + rotate);

    thumb = SVGDoc.getElementById(thumb_id).cloneNode(true);
    thumb.setAttributeNS(null, "id", serial);
    body.appendChild(thumb);

	body.addEventListener("mousedown", SliderDown,  false);
	body.addEventListener("mouseup",   SliderUp,    false);
	body.addEventListener("mousemove", SliderMove,  false);
	body.addEventListener("click",     SliderClick, false);

	this.body  = body;
	this.thumb = thumb;
    this.parent.appendChild(body);
};


/*****	set methods	*****/

/*****
*
*	set_min
*
*****/
Slider.prototype.set_min = function(min) {
	this.min = min;
	if (this.min < this.max) {
		if (this.value < min) this.value = min;
	} else {
		if (this.value < max) this.value = max;
	}
	this.set_value(this.value, true);
};


/*****
*
*	set_max
*
*****/
Slider.prototype.set_max = function(max) {
	this.max = max;
	if (this.min < this.max) {
		if (this.value > max) this.value = max;
	} else {
		if (this.value > min) this.value = min;
	}
	this.set_value(this.value, true);
};


/*****
*
*	set_minmax
*
*****/
Slider.prototype.set_minmax = function(min, max) {
	this.min = min;
	this.max = max;
	if (this.min < this.max) {
		if (this.value < min) this.value = min;
	} else {
		if (this.value < max) this.value = max;
	}
	if (this.min < this.max) {
		if (this.value > max) this.value = max;
	} else {
		if (this.value > min) this.value = min;
	}
	this.set_value(this.value, true);
};


/*****
*
*	set_value
*
*****/
Slider.prototype.set_value = function(value, call_callback) {
	var range    = this.max - this.min;
	var position = ( value - this.min ) / range * this.size;

	this.value = value;
	this.thumb.setAttributeNS(null, "transform", "translate(" + position + ", 0)");

	if (call_callback && this.callback) this.callback(value);
};


/*****
*
*	set_position
*
*****/
Slider.prototype.set_position = function(position, call_callback) {
	var range = this.max - this.min;
	var value = this.min + position / this.size * range;

	this.thumb.setAttributeNS(null, "transform", "translate(" + position + ", 0)");
	this.value = value;
	if (call_callback && this.callback) this.callback(value);
};


/*****
*
*	find_position
*
*****/
Slider.prototype.find_position = function(event) {
	var x_delta  = event.getClientX() - this.x;
	var y_delta  = event.getClientY() - this.y;
	var s_angle  = this.direction * Math.PI / 180;
	var t_angle  = 5*Math.PI/2 - Math.atan2(x_delta, y_delta);
	var angle    = t_angle - s_angle;
	var length   = Math.sqrt( x_delta*x_delta + y_delta*y_delta );
	var position = length * Math.cos(angle);

	if (position < 0) {
		this.set_position(0, true);
	} else if (position > this.size) {
		this.set_position(this.size, true);
	} else {
		this.set_position(position, true);
	}
};


/*****	Event Handlers	*****/

/*****
*
*	SliderDown
*
*****/
function SliderDown(event) {
	var slider = Slider.Find_Slider(event.getTarget().getParentNode());

	slider.active = true;
};


/*****
*
*	SliderUp
*
*****/
function SliderUp(event) {
	var slider = Slider.Find_Slider(event.getTarget().getParentNode());

	slider.active = false;
};


/*****
*
*	SliderMove
*
*****/
function SliderMove(event) {
	var slider = Slider.Find_Slider(event.getTarget().getParentNode());

	if (slider.active) slider.find_position(event);
};


/*****
*
*	SliderClick
*
*****/
function SliderClick(event) {
	var slider = Slider.Find_Slider(event.getTarget().getParentNode());
	
	slider.find_position(event);
};


/*****	Class Methods	*****/

/*****
*
*	Find_Slider
*
*****/
Slider.Find_Slider = function(slider) {
	var result = null;
	var id     = slider.getAttributeNS(null, "id") + "";
	var match  = id.match(/(\d+)$/);

	if (match != null) {
		var index = match[1];
		result = Slider.sliders[index];
	}

	return result;
};
